#include <JTC/JTC.h>
#include "cout.h"
#include "Ordine.h"
#include "Buffer.h"
#include <sstream>

class Pizzeria : public JTCMonitor
{
public:

	Buffer bo;
	Buffer bp;
	int codiceOrdine;


	Pizzeria(): bo(5),bp(5),codiceOrdine(0)
	{
	}

	Ordine getOrdine()
	{
		JTCSynchronized sync (*this);
		Ordine ordine;

		while(bo.isEmpty())
			wait();

		ordine=bo.getFIFO();

		notifyAll();//per svegliare i clienti in attesa di poter ordinare

	    return ordine;
        }

	int  putOrdine(int codicePizza, int quantita)
	{
		JTCSynchronized sync (*this);

		while(bo.isFull())
			wait();

		bo.putFIFO(codicePizza,codiceOrdine,quantita);

		codiceOrdine++;

		notifyAll();//per svegliare i pizzaioli in attesa di ricevere una ordinazione

		return (codiceOrdine-1);
	}

	void putPizze(Ordine ordine)
	{
		JTCSynchronized sync (*this);

		while(bp.isFull())
			wait();

		bp.put(ordine.codicePizza,ordine.codiceOrdine,ordine.quantita);

		notifyAll();//per svegliare i clienti in attesa di prelevare le pizze
	}

	void getPizze(int codiceO)
	{
		JTCSynchronized sync (*this);

		while(!bp.isPresent(codiceO))
			wait();

		bp.clean(codiceO);

		notifyAll();//per svegliare i pizzaioli che devono consegnare altre pizze
	}
};


Pizzeria pizzeria;

class Pizzaiolo: public JTCThread{

int id;

public:
	Pizzaiolo(int i){id=i;}

	virtual void run()
	{
		while(true)
		{
			int tempoDiPreparazione;

			Ordine ordine;

			//stringstream  usato per ottenere una intera stringa da usare in una unica chiamata di coutSync
            stringstream ss1;		
			ss1<<"Il pizzaiolo "<<id<<" verifica la presenza di una ordinazione...\n";
			coutSync<<ss1.str().c_str();
	        
			ordine=pizzeria.getOrdine();

			tempoDiPreparazione=ordine.quantita;

			stringstream ss2;
            ss2<<"Il pizzaiolo "<<id<<" ha prelevato l'ordine numero "<<ordine.codiceOrdine<<" e prepara le pizze \n";
			coutSync<<ss2.str().c_str();
			
			sleep(tempoDiPreparazione*1000);

			stringstream ss3;
            ss3<<"Il pizzaiolo "<<id<<" ha preparato le pizze per l'ordine numero "<<ordine.codiceOrdine<<" \n";	   
            coutSync<<ss3.str().c_str();
			

			pizzeria.putPizze(ordine);

			sleep(rand() % 500);

		}
	}
};

class Cliente: public JTCThread{

int id;

public:
	Cliente(int i){id=i;}

	virtual void run()
	{
      		while(true)
		{

			int numeroPizze=rand()%7+1;
			int codicePizza=rand()%10+1;
	
			stringstream ss1;		
			ss1<<"Il cliente "<<id<<" entra in pizzeria e prova ad ordinare delle pizze \n";
			coutSync<<ss1.str().c_str();
			
		
			int codiceOrdine=pizzeria.putOrdine(codicePizza,numeroPizze);

			stringstream ss2;
			ss2<<"Il cliente "<<id<<" aspetta le pizze con codice d'ordine numero "<<codiceOrdine<<" \n";
			coutSync<<ss2.str().c_str();
					
			sleep(numeroPizze*1000);

			pizzeria.getPizze(codiceOrdine);
		
			stringstream ss3;
			ss3<<"Il cliente "<<id<<" ha preso le pizze con codice d'ordine numero "<<codiceOrdine<<"\n";
			coutSync<<ss3.str().c_str();
            
			
			sleep(rand() % 2000);
		}
	}
};


int main()
	{

		JTCInitialize initialize;

		Pizzaiolo *p[3];
		Cliente *c[10];

		for (int i = 0; i <3; i++) {
		 p[i] = new Pizzaiolo(i);
		 p[i]->start();
	    }

	    for (int j = 0; j <10; j++) {
		 c[j] = new Cliente(j);
		 c[j]->start();
	    }

	return 0;
};
